use 5.008;

use strict;
use warnings;

{
    altitude	=> {
	title	=> 'Batman',
    },
    event	=> {
	table	=> [ qw{ Larry Moe Shemp Curley } ],
    },
};

__END__

=head1 NAME

fu_BAR - Define fictional locale fu_BAR.

=head1 SYNOPSIS

 No user servicable parts inside.

=head1 DESCRIPTION

This Perl module is part of the
L<Astro::App::Satpass2|Astro::App::Satpass2> locale system, used for
testing. It is private to this distribution, and can be changed or
removed without notice.

=head1 SUBROUTINES

None.

=head1 SEE ALSO

L<Astro::App::Satpass2::Locale|Astro::App::Satpass2::Locale>

L<Astro::App::Satpass2::Locale::C|Astro::App::Satpass2::Locale::C>

=head1 SUPPORT

Support is by the author. Please file bug reports at
L<https://rt.cpan.org>, or in electronic mail to the author.

=head1 AUTHOR

Thomas R. Wyant, III F<wyant at cpan dot org>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2014-2020 by Thomas R. Wyant, III

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl 5.10.0. For more details, see the full text
of the licenses in the directory LICENSES.

This program is distributed in the hope that it will be useful, but
without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut

# ex: set textwidth=72 :
